@@ -14,6 +14,7 @@
 # END HEADER
 
 import math
+import sys
 import time
 from collections import defaultdict
 from contextlib import contextmanager
@@ -40,7 +41,7 @@ from hypothesis.internal.conjecture.datatree import (
     PreviouslyUnseenBehaviour,
     TreeRecordingObserver,
 )
-from hypothesis.internal.conjecture.junkdrawer import clamp
+from hypothesis.internal.conjecture.junkdrawer import clamp, stack_depth_of_caller
 from hypothesis.internal.conjecture.pareto import NO_SCORE, ParetoFront, ParetoOptimiser
 from hypothesis.internal.conjecture.shrinker import Shrinker, sort_key
 from hypothesis.internal.healthcheck import fail_health_check
@@ -133,6 +134,10 @@ class ConjectureRunner:
         # executed test case.
         self.__data_cache = LRUReusedCache(CACHE_SIZE)
 
+        # We ensure that the test has this much stack space remaining, no matter
+        # the size of the stack when called, to de-flake RecursionErrors (#2494).
+        self.__recursion_limit = sys.getrecursionlimit()
+
     @contextmanager
     def _log_phase_statistics(self, phase):
         self.stats_per_test_case.clear()
@@ -156,9 +161,21 @@ class ConjectureRunner:
 
     def __stoppable_test_function(self, data):
         """Run ``self._test_function``, but convert a ``StopTest`` exception
-        into a normal return.
+        into a normal return and avoid raising Flaky for RecursionErrors.
         """
+        depth = stack_depth_of_caller()
+        # Because we add to the recursion limit, to be good citizens we also add
+        # a check for unbounded recursion.  The default limit is 1000, so this can
+        # only ever trigger if something really strange is happening and it's hard
+        # to imagine an intentionally-deeply-recursive use of this code.
+        assert depth <= 1000, (
+            "Hypothesis would usually add %d to the stack depth of %d here, "
+            "but we are already much deeper than expected.  Aborting now, to "
+            "avoid extending the stack limit in an infinite loop..."
+            % (self.__recursion_limit, depth)
+        )
         try:
+            sys.setrecursionlimit(depth + self.__recursion_limit)
             self._test_function(data)
         except StopTest as e:
             if e.testcounter == data.testcounter:
@@ -170,6 +187,8 @@ class ConjectureRunner:
                 # need to re-raise it so that it will eventually reach the
                 # correct engine.
                 raise
+        finally:
+            sys.setrecursionlimit(self.__recursion_limit)
 
     def test_function(self, data):
         assert isinstance(data.observer, TreeRecordingObserver)
